@@ -1,9 +1,9 @@
 #
-# $Id: LWP.pm,v 1.66 1998/01/20 15:11:55 aas Exp $
+# $Id: LWP.pm,v 1.82.2.2 1998/09/11 11:12:09 aas Exp $
 
 package LWP;
 
-$VERSION = "5.18_05";
+$VERSION = "5.40_01";
 sub Version { $VERSION; }
 
 require 5.004;
@@ -72,19 +72,10 @@ Supports access through proxy servers.
 
 =item *
 
-URL handling (both absolute and relative URLs are supported).
-
-=item *
-
 A parser for F<robots.txt> files and a framework for constructing robots.
 
 =item *
 
-An experimental HTML parser and formatters (for PostScript and plain
-text).
-
-=item *
-
 The library can cooperate with Tk.  A simple Tk-based GUI browser
 called 'tkweb' is distributed with the Tk extension for perl.
 
@@ -189,7 +180,7 @@ The B<content> is an arbitrary amount of data.
 
 =head2 The Response Object
 
-The request object has the class name C<HTTP::Response> in
+The response object has the class name C<HTTP::Response> in
 libwww-perl.  The main attributes of objects of this class are:
 
 =over 3
@@ -428,7 +419,8 @@ want this).
 The normal file retrievals, the "Content-Type" is guessed based on the
 file name suffix. See L<LWP::MediaTypes>.
 
-The "If-Modified-Since" header is not honored yet.
+The "If-Modified-Since" request header works for servers that implement
+the MDTM command.  It will probably not work for directory listings though.
 
 Example:
 
@@ -530,20 +522,9 @@ library. Indentation shows class inheritance.
    HTTP::Response   -- HTTP response
  HTTP::Daemon       -- A HTTP server class
 
- URI::URL           -- Uniform Resource Locators
-
  WWW::RobotRules    -- Parse robots.txt files
    WWW::RobotRules::AnyDBM_File -- Persistent RobotRules
 
- HTML::Parser       -- Parse HTML documents
-   HTML::TreeBuilder-- Build a HTML syntax tree
-   HTML::HeadParser -- Parse the <HEAD> section of a HTML document
-   HTML::LinkExtor  -- Extract links from a HTML document
- HTML::Element      -- Building block for the HTML::TreeBuilder
- HTML::Formatter    -- Convert HTML syntax trees to readable formats
-   HTML::FormatText -- Output is plain text
-   HTML::FormatPS   -- Output is PostScript
-
 The following modules provide various functions and definitions.
 
  LWP                -- This file.  Library version number and documentation.
@@ -553,17 +534,8 @@ The following modules provide various functions and definitions.
  HTTP::Status       -- HTTP status code (200 OK etc)
  HTTP::Date         -- Date parsing module for HTTP date formats
  HTTP::Negotiate    -- HTTP content negotiation calculation
- HTML::Entities     -- Expand or unexpand entities in HTML text
  File::Listing      -- Parse directory listings
 
-The following modules does not have much to do with the World Wide
-Web, but are included just because I am lazy and did not bother to
-make separate distributions for them.  Regard them as bonus, provided
-free for your pleasure.
-
- Font::AFM          -- Parse Adobe Font Metric files
- File::CounterFile  -- Persistent counter class
-
 
 =head1 MORE DOCUMENTATION
 
@@ -575,9 +547,8 @@ are implemented.
 
 =head1 BUGS
 
-The library can not handle multiple simultaneous requests yet.  The
-HTML:: modules are still experimental.  Also, check out what's left in
-the TODO file.
+The library can not handle multiple simultaneous requests yet.  Also,
+check out what's left in the TODO file.
 
 =head1 ACKNOWLEDGEMENTS
 
