@@ -1,5 +1,5 @@
 #
-# $Id: http.pm,v 1.38 1998/01/20 14:22:44 aas Exp $
+# $Id: http.pm,v 1.43.2.1 1998/10/12 10:21:57 aas Exp $
 
 package LWP::Protocol::http;
 
@@ -77,7 +77,8 @@ sub request
     else {
 	$host = $url->host;
 	$port = $url->port;
-	$fullpath = $url->full_path;
+	$fullpath = $url->path_query;
+	$fullpath = "/" unless length $fullpath;
     }
 
     # connect to remote site
@@ -88,67 +89,61 @@ sub request
 
     my $request_line = "$method $fullpath HTTP/1.0$CRLF";
 
+    my $h = $request->headers->clone;
+    my $cont_ref = $request->content_ref;
+    $cont_ref = $$cont_ref if ref($$cont_ref);
+    my $ctype = ref($cont_ref);
+
     # If we're sending content we *have* to specify a content length
     # otherwise the server won't know a messagebody is coming.
-    my $content = $request->content;
-
-    # All this mess because we want to support content as both scalar,
-    # ref to scalar and ref to code.
-    my $contRef;
-    if (defined $content) {
-	$contRef = ref($content) ? $content : \$content;
-	if (ref($contRef) eq 'SCALAR') {
-	    $request->header('Content-Length' => length $$contRef)
-	        if length $$contRef;
-	} elsif (ref($contRef) eq 'CODE') {
-	    die 'No Content-Length header for request with code content'
-	      unless $request->header('Content-Length');
-	} else {
-	    my $type = ref($contRef);
-	    die "Illegal content type ($type) in request";
-	}
+    if ($ctype eq 'CODE') {
+	die 'No Content-Length header for request with dynamic content'
+	    unless defined($h->header('Content-Length')) ||
+		   $h->content_type =~ /^multipart\//;
+	# For HTTP/1.1 we could have used chunked transfer encoding...
+    } else {
+	$h->header('Content-Length' => length $$cont_ref)
+	        if defined($$cont_ref) && length($$cont_ref);
     }
-
+    
     # HTTP/1.1 will require us to send the 'Host' header, so we might
     # as well start now.
-    {
-	my $host = $url->netloc;
-	$host =~ s/^([^\@]*)\@//;  # get rid of potential "user:pass@"
-	$request->header('Host' => $host);
+    my $hhost = $url->authority;
+    $hhost =~ s/^([^\@]*)\@//;  # get rid of potential "user:pass@"
+    $h->header('Host' => $hhost) unless defined $h->header('Host');
 
-	# add authorization header if we need them
-	if (defined($1) && not $request->header('Authorization')) {
-	    $request->authorization_basic($url->user, $url->password);
-	}
+    # add authorization header if we need them.  HTTP URLs do
+    # not really support specification of user and password, but
+    # we allow it.
+    if (defined($1) && not $h->header('Authorization')) {
+	$h->authorization_basic($url->user, $url->password);
     }
 
-    # we always assume that we are writeable
-    my $buf = $request_line . $request->headers_as_string($CRLF) . $CRLF;
-    {
-	die "write timeout" if $timeout && !$sel->can_write($timeout);
-	my $n = $socket->syswrite($buf, length($buf));
-	die $! unless defined($n);
-	die "short write" unless $n == length($buf);
-	LWP::Debug::conns($buf);
-    }
-    if (defined $content) {
-	if (ref($contRef) eq 'CODE') {
-	    while ( ($buf = &$contRef()), defined($buf) && length($buf)) {
-		die "write timeout" if $timeout && !$sel->can_write($timeout);
-		my $n = $socket->syswrite($buf, length($buf));
-		die $! unless defined($n);
-		die "short write" unless $n == length($buf);
-		LWP::Debug::conns($buf);
-	    }
-	} else {
+    my $buf = $request_line . $h->as_string($CRLF) . $CRLF;
+    my $n;  # used for return value from syswrite/sysread
+
+    die "write timeout" if $timeout && !$sel->can_write($timeout);
+    $n = $socket->syswrite($buf, length($buf));
+    die $! unless defined($n);
+    die "short write" unless $n == length($buf);
+    LWP::Debug::conns($buf);
+
+    if ($ctype eq 'CODE') {
+	while ( ($buf = &$cont_ref()), defined($buf) && length($buf)) {
 	    die "write timeout" if $timeout && !$sel->can_write($timeout);
-	    my $n = $socket->syswrite($$contRef, length($$contRef));
+	    $n = $socket->syswrite($buf, length($buf));
 	    die $! unless defined($n);
-	    die "short write" unless $n == length($$contRef);
+	    die "short write" unless $n == length($buf);
 	    LWP::Debug::conns($buf);
 	}
+    } elsif (defined($$cont_ref) && length($$cont_ref)) {
+	die "write timeout" if $timeout && !$sel->can_write($timeout);
+	$n = $socket->syswrite($$cont_ref, length($$cont_ref));
+	die $! unless defined($n);
+	die "short write" unless $n == length($$cont_ref);
+	LWP::Debug::conns($buf);
     }
-
+    
     # read response line from server
     LWP::Debug::debug('reading response');
 
@@ -160,7 +155,7 @@ sub request
     while (1) {
 	{
 	    die "read timeout" if $timeout && !$sel->can_read($timeout);
-	    my $n = $socket->sysread($buf, $size, length($buf));
+	    $n = $socket->sysread($buf, $size, length($buf));
 	    die $! unless defined($n);
 	    die "unexpected EOF before status line seen" unless $n;
 	    LWP::Debug::conns($buf);
@@ -175,11 +170,11 @@ sub request
 
 	    # ensure that we have read all headers.  The headers will be
 	    # terminated by two blank lines
-	    while ($buf !~ /\015?\012\015?\012/) {
+	    until ($buf =~ /^\015?\012/ || $buf =~ /\015?\012\015?\012/) {
 		# must read more if we can...
 		LWP::Debug::debug("need more header data");
 		die "read timeout" if $timeout && !$sel->can_read($timeout);
-		my $n = $socket->sysread($buf, $size, length($buf));
+		$n = $socket->sysread($buf, $size, length($buf));
 		die $! unless defined($n);
 		die "unexpected EOF before all headers seen" unless $n;
 		#LWP::Debug::conns($buf);
