@@ -1,4 +1,4 @@
-# $Id: UserAgent.pm,v 1.57 1998/01/06 10:20:38 aas Exp $
+# $Id: UserAgent.pm,v 1.63.2.2 1998/10/12 10:15:17 aas Exp $
 
 package LWP::UserAgent;
 use strict;
@@ -92,13 +92,10 @@ use vars qw(@ISA $VERSION);
 
 require LWP::MemberMixin;
 @ISA = qw(LWP::MemberMixin);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.57 $ =~ /(\d+)\.(\d+)/);
-
-
-require URI::URL;
-require HTTP::Request;
-require HTTP::Response;
+$VERSION = sprintf("%d.%02d", q$Revision: 1.63.2.2 $ =~ /(\d+)\.(\d+)/);
 
+use HTTP::Request ();
+use HTTP::Response ();
 use HTTP::Date ();
 
 use LWP ();
@@ -106,7 +103,6 @@ use LWP::Debug ();
 use LWP::Protocol ();
 
 use Carp ();
-
 use AutoLoader ();
 *AUTOLOAD = \&AutoLoader::AUTOLOAD;  # import the AUTOLOAD method
 
@@ -263,19 +259,26 @@ sub request
 	my $referral = $request->clone;
 
 	# And then we update the URL based on the Location:-header.
-	# Some servers erroneously return a relative URL for redirects,
-	# so make it absolute if it not already is.
-	my $referral_uri = (URI::URL->new($response->header('Location'),
-					  $response->base))->abs();
+	my $referral_uri = $response->header('Location');
+	{
+	    # Some servers erroneously return a relative URL for redirects,
+	    # so make it absolute if it not already is.
+	    local $URI::ABS_ALLOW_RELATIVE_SCHEME = 1;
+	    my $base = $response->base;
+	    $referral_uri = $HTTP::URI_CLASS->new($referral_uri, $base)
+		            ->abs($base);
+	}
 
 	$referral->url($referral_uri);
 
 	return $response unless $self->redirect_ok($referral);
 
 	# Check for loop in the redirects
+	my $count = 0;
 	my $r = $response;
 	while ($r) {
-	    if ($r->request->url->as_string eq $referral_uri->as_string) {
+	    if (++$count > 13 ||
+                $r->request->url->as_string eq $referral_uri->as_string) {
 		$response->header("Client-Warning" =>
 				  "Redirect loop detected");
 		return $response;
@@ -304,6 +307,9 @@ sub request
 	my $scheme = lc(shift(@$challenge));
 	shift(@$challenge); # no value
 	$challenge = { @$challenge };  # make rest into a hash
+	for (keys %$challenge) {       # make sure all keys are lower case
+	    $challenge->{lc $_} = delete $challenge->{$_};
+	}
 
 	unless ($scheme =~ /^([a-z]+(?:-[a-z]+)*)$/) {
 	    $response->header("Client-Warning" => 
@@ -395,9 +401,9 @@ sub get_basic_credentials
     my($self, $realm, $uri, $proxy) = @_;
     return if $proxy;
 
-    my $netloc = $uri->netloc;
-    if (exists $self->{'basic_authentication'}{$netloc}{$realm}) {
-	return @{ $self->{'basic_authentication'}{$netloc}{$realm} };
+    my $host_port = $uri->host_port;
+    if (exists $self->{'basic_authentication'}{$host_port}{$realm}) {
+	return @{ $self->{'basic_authentication'}{$host_port}{$realm} };
     }
 
     return (undef, undef);
@@ -460,9 +466,14 @@ sub cookie_jar { shift->_elem('cookie_jar',@_); }
 sub parse_head { shift->_elem('parse_head',@_); }
 sub max_size   { shift->_elem('max_size',  @_); }
 
-# depepreciated
-sub use_alarm  { shift->_elem('use_alarm', @_); }
+# depreciated
 sub use_eval   { shift->_elem('use_eval',  @_); }
+sub use_alarm
+{
+    Carp::carp("LWP::UserAgent->use_alarm(BOOL) is a no-op")
+	if @_ > 1 && $^W;
+    "";
+}
 
 
 # Declarations of AutoLoaded methods
@@ -502,7 +513,7 @@ sub clone
 
 You can use this method to query if the library currently support the
 specified C<scheme>.  The C<scheme> might be a string (like 'http' or
-'ftp') or it might be an URI::URL object reference.
+'ftp') or it might be an URI object reference.
 
 =cut
 
@@ -510,13 +521,14 @@ sub is_protocol_supported
 {
     my($self, $scheme) = @_;
     if (ref $scheme) {
-	# assume we got a reference to an URI::URL object
-	$scheme = $scheme->abs->scheme;
+	# assume we got a reference to an URI object
+	$scheme = $scheme->scheme;
     } else {
 	Carp::croak("Illeal scheme '$scheme' passed to is_protocol_supported")
 	    if $scheme =~ /\W/;
 	$scheme = lc $scheme;
     }
+    local($SIG{__DIE__});  # protect agains user defined die handlers
     return LWP::Protocol::implementor($scheme);
 }
 
@@ -568,6 +580,11 @@ sub mirror
 	    }
 	    rename($tmpfile, $file) or
 		die "Cannot rename '$tmpfile' to '$file': $!\n";
+
+	    if (my $lm = $response->last_modified) {
+		# make sure the file has the same last modification time
+		utime $lm, $lm, $file;
+	    }
 	}
     } else {
 	unlink($tmpfile);
@@ -665,35 +682,23 @@ sub no_proxy {
 sub _need_proxy
 {
     my($self, $url) = @_;
+    $url = $HTTP::URI_CLASS->new($url) unless ref $url;
 
-    $url = new URI::URL($url) unless ref $url;
-
-    LWP::Debug::trace("($url)");
-
-    # check the list of noproxies
-
-    if (@{ $self->{'no_proxy'} }) {
-	my $host = $url->host;
-	return undef unless defined $host;
-	my $domain;
-	for $domain (@{ $self->{'no_proxy'} }) {
-	    if ($host =~ /$domain$/) {
-		LWP::Debug::trace("no_proxy configured");
-		return undef;
+    my $scheme = $url->scheme || return;
+    if (my $proxy = $self->{'proxy'}{$scheme}) {
+	if (@{ $self->{'no_proxy'} }) {
+	    if (my $host = eval { $url->host }) {
+		for my $domain (@{ $self->{'no_proxy'} }) {
+		    if ($host =~ /\Q$domain\E$/) {
+			LWP::Debug::trace("no_proxy configured");
+			return;
+		    }
+		}
 	    }
 	}
+	LWP::Debug::debug("Proxied to $proxy");
+	return $HTTP::URI_CLASS->new($proxy);
     }
-
-    # Currently configured per scheme.
-    # Eventually want finer granularity
-
-    my $scheme = $url->scheme;
-    if (exists $self->{'proxy'}{$scheme}) {
-
-	LWP::Debug::debug('Proxied');
-	return new URI::URL($self->{'proxy'}{$scheme});
-    }
-
     LWP::Debug::debug('Not proxied');
     undef;
 }
@@ -709,7 +714,7 @@ F<lwp-mirror> for examples of usage.
 
 =head1 COPYRIGHT
 
-Copyright 1995-1997 Gisle Aas.
+Copyright 1995-1998 Gisle Aas.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
